Map = class("Map")
Map.static.Size = 20
Map.static.Height = 6
Map.static.Speed = 0.1
Map.static.BlockSize = 30

function Map:getHighestPoint(x, y)
	local block = self:getHighestBlock(x, y)
	if block then
		return block.z
	else
		return 0
	end
end

function Map:getHeightBelow(obj)
	local block = self:getBlockBelow(obj)
	if block then
		return block.z
	else
		return 0
	end
end

function Map:getBlockBelow(obj)
	for i = #self.content, 1, -1 do
		local v = self.content[i]
		if v.x == obj.x and v.y == obj.y and v.z < obj.z and v.layer == obj.layer then
			return v
		end
	end
	return nil
end

function Map:moveLeft(obj)
	if obj.x == 1 then return end
	obj.x = obj.x - 1
	self:avoidCollision(obj)
end

function Map:moveForward(obj)
	if obj.y == 1 then return end
	obj.y = obj.y - 1
	self:avoidCollision(obj)
end

function Map:moveBackward(obj)
	if obj.y == Map.Size then return end
	obj.y = obj.y + 1
	self:avoidCollision(obj)
end

function Map:moveRight(obj)
	if obj.x == Map.Size then return end
	obj.x = obj.x + 1
	self:avoidCollision(obj)
end

function Map:moveUp(obj)
	if obj.z == Map.Height then return end
	obj.z = obj.z + 1
end

function Map:getHighestBlock(x, y)
	for i = #self.content, 1, -1 do
		local v = self.content[i]
		if v.x == x and v.y == y then
			return v
		end
	end
	return nil
end

function Map:insert(obj)
	if obj.z == -1 then
		obj.z = self:getHighestPoint(obj.x, obj.y) + 1
	end
	for i, v in ipairs(self.content) do
		if v.x == obj.x and v.y == obj.y and v.z == obj.z and v.layer == obj.layer then
			self:delete(v)
			break
		end
	end
	table.insert(self.content, obj)
end

function Map:delete(obj)
	for i, v in ipairs(self.content) do
		if v == obj then
			table.remove(self.content, i)
			break
		end
	end
end

function Map:initialize()
	self.content = {}
	
	self.viewer = nil	
	self.clock = 0
	
	-- only move task for now
	self.task = {}
end

function Map:doMove()
	for i, v in ipairs(self.task) do
		if v.type == "move" then
			local obj = v.obj
			local x = v.x
			local y = v.y
			
			local diffx = math.abs(obj.x - x)
			local diffy = math.abs(obj.y - y)
			if diffx > diffy then
				--move x
				if obj.x > x then
					self:moveLeft(obj)
				end
				
				if obj.x < x then
					self:moveRight(obj)
				end
			else
				--move y
				if obj.y > y then
					self:moveForward(obj)
				end
				
				if obj.y < y then
					self:moveBackward(obj)
				end
			end
			
			-- mission complete
			if obj.x == x and obj.y == y then
				table.remove(self.task, i)
			end
		end
	end
end

function Map:move(obj, x, y)
	for i, v in ipairs(self.task) do
		if v.type == "move" and v.obj == obj then
			table.remove(self.task, i)
		end
	end
	table.insert(self.task, {
		type = "move",
		obj = obj,
		x = x,
		y = y,
	})
end

function Map:standardDensity()
	return math.random(math.ceil(Map.Size/8), math.ceil(Map.Size/4))
end

function Map:addTree()
	local x = math.random(1, Map.Size)
	local y = math.random(1, Map.Size)
	self:insert(WorldObj:new(2, x, y, self:getHighestPoint(x, y) + 1))
end

function Map:addStone()
	local x = math.random(1, Map.Size)
	local y = math.random(1, Map.Size)
	self:insert(WorldObj:new(3, x, y, self:getHighestPoint(x, y) + 1))
end

function Map:searchByCoord(x, y ,z)
	for i, v in ipairs(self.content) do
		if v.x == x and v.y == y and v.z == z then
			return v
		end
	end
	return nil
end

function Map:generate()
	for i = 1, Map.Size do
		for j = 1, Map.Size do
			self:insert(WorldObj:new(1, i, j, 1))
		end
	end
	
	for i = 1, self:standardDensity() do
		self:addMountain()
	end
	
	for i = 1, self:standardDensity() do
		self:addStone()
		self:addTree()
	end
	
	for i = 1, self:standardDensity() do
		self:addWater()
	end
end

function Map:hasSomething(x, y, z)
	return not not self:searchByCoord(x, y, z)
end

function Map:addWater()
	local x = math.random(1, Map.Size)
	local y = math.random(1, Map.Size)
	if self:getHighestPoint(x, y) ~= 1 then
		return
	end
	
	local range = math.random(3, Map.Height)
	
	for i = x - range, x + range do
		for j = y - range, y + range do
			local distance = WorldObj:distance(x, y, i, j)
			if Map:inMap(i, j, 1) and math.random(0, math.log(distance)) == 0 and self:getHighestPoint(i, j) == 1  then
				b = self:getHighestBlock(i, j)
				b.skin = 4
			end
		end
	end
end

function Map.static:inMap(x, y, z)
	return 1 <= x and x <= Map.Size and 1 <= y and y <= Map.Size and 1 <= z and z <= Map.Height
end

function Map:addMountain()
	local x = math.random(1, Map.Size)
	local y = math.random(1, Map.Size)
	--print (x..",".. y)
	local range = self:standardDensity()
	
	for i = x - range, x + range do
		for j = y - range, y + range do
			local distance = WorldObj:distance(x, y, i, j)
			local myheight = range - distance
			if Map:inMap(i, j, 1) and myheight >= 2 then
				for k = 2, myheight do
					if not self:hasSomething(i, j, k) then
						self:insert(WorldObj:new(1, i, j, k))
					end
				end
			end
		end
	end
end

function Map:covered(obj)
	for i = #self.content, 1, -1 do
		local v = self.content[i]
		if v:cover(obj) then
			return true
		end
	end
	return false
end

function Map:draw(offx, offy)
	--print("===")
	for i, v in ipairs(self.content) do
		--print(v.x, v.y, v.z)
		if self.viewer and 2 + self.viewer.sight + self.viewer.z >= self.viewer:distanceFrom(v) then
			v:draw(offx, offy)
		end
	end
	
	if self.viewer then
		for i, v in ipairs(self.content) do
			local distance = self.viewer:distanceFrom(v)
			if not (v.x == self.viewer.x and v.y == self.viewer.y) and distance <= self.viewer.range and v.z == self.viewer.z - 1 and v.weight > 0 and not self:covered(v) then
				local range = WorldObj:new(7, v.x, v.y, v.z)
				range:draw(offx, offy)
			elseif distance > self.viewer.sight + self.viewer.z then
				local blur = WorldObj:new(8, v.x, v.y, v.z)
				blur:draw(offx, offy)
			end
		end
		local arrow = WorldObj:new(6, self.viewer.x, self.viewer.y, self.viewer.z + 3)
		arrow:draw(offx, offy)
	end

	--print("===")
end

function Map:near(obj, distance)
	local blocks = {}
	for i, v in ipairs(self.content) do
		if obj ~= v and obj:distanceFrom(v) <= distance then
			table.insert(blocks, v)
		end
	end
	return blocks
end

function Map:toCoord(px, py, offx, offy)
	for i, v in ipairs(self.content) do
		if v:clicked(px, py, offx, offy) then
			return v.x, v.y, v.z
		end
	end
	return -1, -1, -1
end

function Map:avoidCollision(obj)
	if obj.weight == 0 then return end
	for i, v in ipairs(self.content) do
		if obj ~= v and obj.x == v.x and obj.y == v.y and obj.z - v.z < 1 and obj.z - v.z >= 0 and v.weight > 0 and obj.layer == v.layer then
			obj.z = v.z + 1
			break
		end
	end
end

function Map:applyGravity(obj, gravity)
	local rightHeight = self:getHeightBelow(obj) + 1
	if obj.weight > 0 and obj.z > rightHeight then
		obj.z = obj.z - (5 * gravity * obj.weight)
		if obj.z < rightHeight then
			obj.z = rightHeight
		end
	end
end

function Map:export()
	local state = {}
	for i, v in ipairs(self.content) do
		table.insert(state, {x=v.x, y=v.y, z=v.z, s=v.skin})
	end
	return state
end

function Map:import(state)
	--print(state)
	self.content = {}
	for i, v in ipairs(state) do
		table.insert(self.content, WorldObj:new(v.s, v.x, v.y, v.z))
	end
end

function Map:update(dt)
	if self.content == {} then
		self:generate()
	end
	
	self.clock = self.clock - dt
	if self.clock <= 0 then
		self.clock = Map.Speed
		self:doMove()
	end
	
	for i, v in ipairs(self.content) do
		if v.dead then
			self:delete(v)
		else
			v:update(dt)
			-- gravity
			self:applyGravity(v, dt)
			
			-- collision avoidance
			self:avoidCollision(v)
		end
	end
	table.sort(self.content, inFrontOf)
end
